#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a new line art image framed (using G'MIC filters)

# (c) Dinasset 2023
#
#   History:
#
#   v0.0: 2023-06: Initial version
#   v0.1: 2023-06:added user choice of frame color  and size (by MareroQ)
#         		  + some improvements 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
from gimpfu import *
############################################################################
# Service routine (by MareroQ)
#---------------------------------------------------------------------------
def gmic_rgba_to_rgb(rgba_color):

	r = rgba_color[0]
	g = rgba_color[1]
	b = rgba_color[2]

	# write rgb in correct syntax
	rgb_color = "{0},{1},{2}".format(r,g,b)
	return rgb_color
	
############################################################################
# Main
#---------------------------------------------------------------------------

def LineartFramed (inImage, inLayer,
                    frame, size_fr, color_frame, style,
                    regaincolours, thinlines,
                    doFlatten):
     
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
    
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    
    newImage = pdb.gimp_image_duplicate (inImage)
    newBGo = newImage.layers[0]
    newBGo.name = "new BG original"
    
    # create base for applying chosen 
    baseforeffect = newBGo.copy()
    newImage.add_layer (baseforeffect, 0)
    baseforeffect.name = "base for effect"
    
    # check and -if necessary- adjust mean
    (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (baseforeffect, HISTOGRAM_VALUE,
                                                     0.0, 1.0)
    if mean<117.5:
        #gimp.message ("mean before="+str(mean))
        while mean<100:  
            pdb.gimp_drawable_levels (baseforeffect, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      1.10, 0.0, 1.00, False)
            (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (baseforeffect,
                                                 HISTOGRAM_VALUE, 0.0, 1.0)
        #gimp.message ("mean after="+str(mean))
        baseforeffect.name = baseforeffect.name+" mean adj"
    elif mean>137.5:
        #gimp.message ("mean before="+str(mean))
        while mean>155:  
            pdb.gimp_drawable_levels (baseforeffect, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.90, 0.0, 1.00, False)
            (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (baseforeffect,
                                                 HISTOGRAM_VALUE, 0.0, 1.0)
        #gimp.message ("mean after="+str(mean))
        baseforeffect.name = baseforeffect.name+" mean adj"
    
    # Use some functionalities of G'MIC to assess structure of the image
    
    # G'MIC Auto Balance
    # gcd_auto_balance 30,0,0,1,0
    parms='30,0,0,1,0'
    pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,0,
        "-v - -gcd_auto_balance "+str(parms)+" ")
    baseforeffect.name = baseforeffect.name+" auto-balanced"
    
    # G'MIC Channel_Processing (equalize)
    # fx_channel_processing 0,0,0,0,0,0,100,256,1,0,0,2,0,0,50,50
    parms='0,0,0,0,0,0,100,256,1,0,0,2,0,0,50,50'
    pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,0,
        "-v - -fx_channel_processing "+str(parms)+" ")
    baseforeffect.name = baseforeffect.name+" channels-equalized"
    
    # G'MIC Color Balance
    # fx_balance_gamma 128,128,128,1,0,50,50
    parms='128,128,128,1,0,50,50'
    pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,0,
        "-v - -fx_balance_gamma "+str(parms)+" ")
    baseforeffect.name = baseforeffect.name+" color-balanced"
    
    # G'MIC Normalize Brightness
    # fx_balance_gamma 0,10,0,3,0,0 (dft for dehaze)
    parms='0,10,0,3,0,0'
    pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,0,
        "-v - -gcd_normalize_brightness "+str(parms)+" ")
    baseforeffect.name = baseforeffect.name+" gamma-balanced"
    
    # G'MIC local contrast 
    # afre_localcontrast 1,50
    pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,0,
        "-v - -afre_localcontrast 1,50 ")
    
    # Name the BGlayer to use for styles
    newBG = pdb.gimp_image_get_active_layer (newImage)
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # START APPLYING USER CHOICES
    
    # set defaults mode/opacity
    autolayermode = LAYER_MODE_NORMAL  # default mode
    autolayeropacity = 100.0  # default opacity
    
    # apply chosen GMIC Colors effect
    if style==0:    #Gradient Normalize
        # Apply gradient_norm
        #[G'MIC] fx_gradient_norm 0,0.5,0,100,0,0,50,50
        parms='0,0.5,0,100,0,0,50,50'
    
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)
   
        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_gradient_norm "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Gradient Norm "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)
                                           
    #pdb.gimp_display_new (newImage)
    #return
         
    if style==1:    #Edge (by afre)
        # Apply afre_edge
        #[G'MIC] -afre_edge 0,1,4,0.5,4    
        parms='0,1,4,0.5,4'
    
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)
   
        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -afre_edge "+str(parms))     
        
        LineartLayer.name = "Lineart afre_edge "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)
              
    #pdb.gimp_display_new (newImage)
    #return
        
    if style==2:    #Edges
        # Apply fx_edges
        #[G'MIC] fx_edges 0,5,0,0,50,50
        parms='0,5,0,0,50,50'
    
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)
   
        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_edges "+str(parms))
        
        LineartLayer.name = "Lineart edges Layer "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        #pdb.gimp_drawable_invert (LineartLayer,False)    # already inverted
              
    #pdb.gimp_display_new (newImage)
    #return
        
    if style==3:    #JR_Gradient_Norm
        # Apply jr_gradient_norm
        #[G'MIC] fx_jr_gradient_norm 0,0.45,0,100,0,0,1,1,2,0,0,50,50    
        parms='0,0.45,0,100,0,0,1,1,2,0,0,50,50'
    
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)
        

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_jr_gradient_norm "+str(parms))
           
        LineartLayer.name = "Lineart JR Grad Norm "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
              
    #pdb.gimp_display_new (newImage)
    #return
        
    if style==4:    #Thin_Edges
        # Apply thin_edges
        #[G'MIC] fx_thin_edges 0,10,0,0,50,50
        parms='0,10,0,0,50,50'
        
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)
        
        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_thin_edges "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart: Thin_Edges "+str(parms)        
        
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        #already inverted
              
    #pdb.gimp_display_new (newImage)
    #return
    
    # -----------------------------------------------------------------------------
    # CONVOLVE: various types
    # -----------------------------------------------------------------------------

    if style==5:    #Convolve (Prewitt X)
        # Apply convolve
        #[G'MIC] fx_convolve 5,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50  
        parms='5,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50'
  
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Prewitt X "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
        
    #pdb.gimp_display_new (newImage)
    #return
        
    if style==6:    #Convolve (Prewitt Y)
        # Apply convolve
        # [G'MIC] fx_convolve 6,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50  
        parms='6,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50'
  
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Prewitt Y "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
              
    #pdb.gimp_display_new (newImage)
    #return

    if style==7:    #Convolve (Sobel X)
        # Apply convolve
        # [G'MIC] fx_convolve 7,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50  
        parms='7,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50'
  
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Sobel X "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
              
    #pdb.gimp_display_new (newImage)
    #return
        
    if style==8:    #Convolve (Sobel Y)
        # Apply convolve
        #[G'MIC] fx_convolve 8,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50  
        parms='8,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50'
  
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Sobel Y "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
              
    #pdb.gimp_display_new (newImage)
    #return

    if style==9:    #Convolve (Potinv X)
        # Apply convolve
        # [G'MIC] fx_convolve 9,1,"0,1,0;1,-4,1;0,1,0",0,1,27,0,50,50  
        parms='9,1,"0,1,0;1,-4,1;0,1,0",0,1.0,27,0,50,50'
   
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Potinv X "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
              
    #pdb.gimp_display_new (newImage)
    #return
        
    if style==10:    #Convolve (Potinv Y)
        # Apply convolve
        # [G'MIC] fx_convolve 10,1,"0,1,0;1,-4,1;0,1,0",0,1,27,0,50,50    
        parms='10,1,"0,1,0;1,-4,1;0,1,0",0,1,0,27,50,50'
  
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Potinv Y "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
              
    #pdb.gimp_display_new (newImage)
    #return
        
    if style==11:    #Convolve (Laplacian)
        # Apply convolve
        # [G'MIC] fx_convolve 11,1,"0,1,0;1,-4,1;0,1,0",0,1,27,0,50,50    
        parms='11,1,"0,1,0;1,-4,1;0,1,0",0,1,27,0,50,50'
  
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Laplacian "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)
        #pdb.plug_in_vinvert (newImage, LineartLayer)    
              
    #pdb.gimp_display_new (newImage)
    #return
     
    if style==12:    #Convolve (Cross 1)
        # Apply convolve
        # [G'MIC] fx_convolve 12,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50 
         
        parms='12,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50'
   
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Cross 1 "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)
        #pdb.plug_in_vinvert (newImage, LineartLayer)    #t
              
    #pdb.gimp_display_new (newImage)
    #return
        
    if style==13:    #Convolve (Cross 2)
        # Apply convolve
        #[G'MIC] fx_convolve 13,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50 
        parms='13,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50'
  
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Cross 2 "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
              
    #pdb.gimp_display_new (newImage)
    #return
    
    # ------------------------------
    # CONVOLVE: various types double
    # ------------------------------
                        
    if style==14:    #Convolve (Prewitt X+Y)
        # Apply convolve X
        #[G'MIC] fx_convolve 5,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50  
        parms='5,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50'
        
        LineartLayer1 = newBG.copy()
        newImage.add_layer (LineartLayer1, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer1, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer1.name = "Lineart Convolve Prewitt X "+str(parms)
         
        LineartLayer1.mode = LAYER_MODE_NORMAL
        LineartLayer1.opacity = 100.0
        
        # Apply convolve Y
        # [G'MIC] fx_convolve 6,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50  
        parms='6,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50'
  
        LineartLayer2 = newBG.copy()
        newImage.add_layer (LineartLayer2, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer2, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer2.name = "Lineart Convolve Prewitt Y "+str(parms)
         
        LineartLayer2.mode = LAYER_MODE_LIGHTEN_ONLY
        LineartLayer2.opacity = 100.0
              
        #pdb.gimp_display_new (newImage)
        #return
    
        LineartLayer=pdb.gimp_image_merge_down (newImage,LineartLayer2,0)
        LineartLayer.name = "Lineart Convolve Prewitt X+Y "
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
        
        #pdb.gimp_display_new (newImage)
        #return
        
    if style==15:    #Convolve (Sobel X+Y)
        # Apply convolve X
        #[G'MIC] fx_convolve 7,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50  
        parms='7,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50'
  
        LineartLayer1 = newBG.copy()
        newImage.add_layer (LineartLayer1, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer1, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer1.name = "Lineart Convolve Sobel X "+str(parms)
         
        LineartLayer1.mode = LAYER_MODE_NORMAL
        LineartLayer1.opacity = 100.0
        
        # Apply convolve Y
        #[G'MIC] fx_convolve 8,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50   
        parms='8,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50'
  
        LineartLayer2 = newBG.copy()
        newImage.add_layer (LineartLayer2, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer2, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer2.name = "Lineart Convolve Sobel Y "+str(parms)
         
        LineartLayer2.mode = LAYER_MODE_LIGHTEN_ONLY
        LineartLayer2.opacity = 100.0
              
        #pdb.gimp_display_new (newImage)
        #return
    
        LineartLayer=pdb.gimp_image_merge_down (newImage,LineartLayer2,0)
        LineartLayer.name = "Lineart Convolve Sobel X+Y "
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        #pdb.gimp_display_new (newImage)
        #return
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
              
    #pdb.gimp_display_new (newImage)
    #return

    if style==16:    #Convolve (Potinv X+Y)
        # Apply convolve X
        #[G'MIC] fx_convolve 9,1,"0,1,0;1,-4,1;0,1,0",0,1,27,0,50,50
        parms='9,1,"0,1,0;1,-4,1;0,1,0",0,1,27,0,50,50'
  
        LineartLayer1 = newBG.copy()
        newImage.add_layer (LineartLayer1, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer1, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer1.name = "Lineart Convolve Potinv X "+str(parms)
         
        LineartLayer1.mode = LAYER_MODE_NORMAL
        LineartLayer1.opacity = 100.0
        
        # Apply convolve Y
        # [G'MIC] fx_convolve 10,1,"0,1,0;1,-4,1;0,1,0",0,1,27,0,50,50   
        parms='10,1,"0,1,0;1,-4,1;0,1,0",0,1,0,27,50,50'
  
        LineartLayer2 = newBG.copy()
        newImage.add_layer (LineartLayer2, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer2, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer2.name = "Lineart Convolve Potinv Y "+str(parms)
         
        LineartLayer2.mode = LAYER_MODE_LIGHTEN_ONLY
        LineartLayer2.opacity = 100.0
              
        #pdb.gimp_display_new (newImage)
        #return
    
        LineartLayer=pdb.gimp_image_merge_down (newImage,LineartLayer2,0)
        LineartLayer.name = "Lineart Convolve Potinv X+Y "
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0

        pdb.gimp_drawable_invert (LineartLayer,False)    
        
        #pdb.gimp_display_new (newImage)
        #return
    
    if style==17:    #Convolve (Cross 1+2)
        # Apply convolve X
        #[G'MIC] fx_convolve 12,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50 
        parms='12,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50'
  
        LineartLayer1 = newBG.copy()
        newImage.add_layer (LineartLayer1, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer1, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer1.name = "Lineart Convolve Cross 1 "+str(parms)
         
        LineartLayer1.mode = LAYER_MODE_NORMAL
        LineartLayer1.opacity = 100.0
        
        # Apply convolve Y
        #[G'MIC]  fx_convolve 13,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50 
        parms='13,1,"0,1,0;1,-4,1;0,1,0",0,0.5,27,0,50,50'
  
        LineartLayer2 = newBG.copy()
        newImage.add_layer (LineartLayer2, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer2, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer2.name = "Lineart Convolve Cross 2 "+str(parms)
         
        LineartLayer2.mode = LAYER_MODE_LIGHTEN_ONLY
        LineartLayer2.opacity = 50.0
              
        #pdb.gimp_display_new (newImage)
        #return
    
        LineartLayer=pdb.gimp_image_merge_down (newImage,LineartLayer2,0)
        LineartLayer.name = "Lineart Convolve Cross 1+2 "
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        #pdb.gimp_display_new (newImage)
        #return
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
              
    #pdb.gimp_display_new (newImage)
    #return
    
    # COMMON ******************************************************
    
    # Threshold layer  
    
    ThresholdLayer = pdb.gimp_layer_new_from_visible (newImage,
                                                      newImage,
                                                      "Visible layer for Threshold")
    newImage.add_layer (ThresholdLayer, 0)
       
    #pdb.gimp_display_new (newImage)
    #return
         
    # find mean
    (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (ThresholdLayer,
                                                    HISTOGRAM_VALUE,
                                                    0.0, 1.0)
    mean100 = mean/255
    #gimp.message (str(mean100))
       
    # apply threshold
        
    adjmean=mean100 # forced mean <<<<<<<<<<<<<<<<<<
    
    #gimp.message (str(adjmean))
    
    #pdb.gimp_display_new (newImage)
    #return

    pdb.gimp_drawable_threshold (ThresholdLayer,HISTOGRAM_VALUE,
                                 adjmean, 1.0)  #  
    autolayermode = LAYER_MODE_NORMAL 
    autolayeropacity = 100.0    
        
    #pdb.gimp_display_new (newImage)
    #return
      
    ThresholdLayer.mode = autolayermode
    ThresholdLayer.opacity = autolayeropacity
    
    # added BG desaturated on top
    BGdesat = newBGo.copy()
    newImage.add_layer (BGdesat, 0)
    pdb.gimp_drawable_desaturate (BGdesat, DESATURATE_LUMINANCE)
    BGdesat.mode = LAYER_MODE_LINEAR_BURN   #to adjust B&W values
    BGdesat.name = "Original desaturated"
    BGdesat.opacity = 100.0
    
    prefinal_layer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                      "Pre_Outcome")
    newImage.add_layer (prefinal_layer, 0)
    prefinal_layer.name = "Preliminary Outcome"
    prefinal_layer.mode = LAYER_MODE_NORMAL
        
    #pdb.gimp_display_new (newImage)
    #return
    
    if thinlines==True:
        # apply morpho-dilation
        # fx_morphological 1,2,5,"1,0,1; 0,1,0; 1,0,1",0,0,0,0,0,50,50
        parms='1,2,5,"1,0,1; 0,1,0; 1,0,1",0,0,0,0,0,50,50'
        # double size, to half the dilation
        pdb.gimp_layer_scale (prefinal_layer,
                              prefinal_layer.width*2,
                              prefinal_layer.height*2,
                              True)           
        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_morphological "+str(parms)+" ")     
        # back to original size
        pdb.gimp_layer_scale (prefinal_layer,
                              newBG.width,
                              newBG.height,
                              True)
    if regaincolours==True:
        BGonTop = newBGo.copy()
        newImage.add_layer (BGonTop, 0)
        BGonTop.mode = LAYER_MODE_LCH_COLOR
        BGonTop.opacity = 91.0
        
    #pdb.gimp_display_new (newImage)
    #return
     
    # outcome
    
    outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
    newImage.add_layer (outcome, 0)
    outcome.name = "Outcome (unframed)"
        
    #pdb.gimp_display_new (newImage)
    #return
    
    # G'MIC Auto Balance outcome
    # gcd_auto_balance 30,0,0,1,0
    outcomeb = outcome.copy()
    newImage.add_layer (outcomeb, 0)
    parms='30,0,0,1,0'
    pdb.plug_in_gmic_qt(newImage,outcomeb, 1,0,
        "-v - -gcd_auto_balance "+str(parms)+" ")
    outcomeb.opacity = 30.0
    outcome=pdb.gimp_image_merge_down (newImage,outcomeb,0)
    
    # apply GMIC Frame Blur  
    # fx_frame_blur 10,10,0,5,1,0,128,128,128,0,0,255,255,255,2,2,1,0,0.5,0.5,0
    hex = gmic_rgba_to_rgb(color_frame)
    SH = int(size_fr)
    SV = int(size_fr)
    Round=["0.0","1.0","0.5","0.2","0.1"]	
    Rnd=str(Round[frame])
    XSh = min(10,int(size_fr/6))
    YSh = min(10,int(size_fr/6))
    OuS = min(10,int(size_fr/3))	
    parms=str(SH)+','+str(SV)+',0,5,'+str(Rnd)+',0,128,128,128,0,'
    parms=parms+str(OuS)+','+str(hex)+','+str(XSh)+','+str(YSh)+',1,0,0.5,0.5,0 '	
    pdb.plug_in_gmic_qt(newImage, outcome, 1,2,
        "-v - -fx_frame_blur "+str(parms)+" ")
    Framed = pdb.gimp_image_get_active_layer(newImage)
    Framed.mode = LAYER_MODE_NORMAL
 
    if doFlatten==True:
        pdb.gimp_image_remove_layer (newImage, outcome)
        if regaincolours==True:
            pdb.gimp_image_remove_layer (newImage, BGonTop)
        pdb.gimp_image_remove_layer (newImage, prefinal_layer)
        #pdb.gimp_image_remove_layer (newImage, Iso_layer)
        pdb.gimp_image_remove_layer (newImage, BGdesat)
        pdb.gimp_image_remove_layer (newImage, ThresholdLayer)
        pdb.gimp_image_remove_layer (newImage, LineartLayer)
        pdb.gimp_image_remove_layer (newImage, newBG)
         
    #pdb.gimp_display_new (newImage)
     
    stylename = ["Gradient Normalize","Afre Edge",
                 "Edges","Gradient Norm JR",
                 "Thin_Edges",
                 "Convolve Prewitt X","Convolve Prewitt Y",
                 "Convolve Sobel X","Convolve Sobel Y",
                 "Convolve Potinv X","Convolve Potinv Y",
                 "Convolve Laplacian",
                 "Convolve Cross 1","Convolve Cross 2",
                 "Convolve Prewitt X+Y",
                 "Convolve Sobel X+Y",
                 "Convolve Potinv X+Y",
                 "Convolve Cross 1+2",
                  ]     
    Framed.name = st+oname+" with a framed line art v.1.01 effect, style "
    Framed.name = Framed.name+str(stylename[style])
    if regaincolours==True:
        Framed.name = Framed.name+" (coloured)"
    if thinlines==True:
        Framed.name = Framed.name+" (reducing black strokes)"
        
    Framed2 = Framed.copy()
    newImage.add_layer (Framed2, 0)
    Framed2.mode = LAYER_MODE_OVERLAY
    Framed=pdb.gimp_image_merge_down (newImage,Framed2,0)
    Framed.mode = LAYER_MODE_NORMAL
    Framed.opacity = 100.0
    
        
    pdb.gimp_display_new (newImage)
    return
             
### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/SimpleFilters/"
copyrightYear="2023"
desc="Line Art Framed v.1.01"

register(
    "LineartFramed",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    "Line Art Framed v.1.01 ...",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "drawable", "Input drawable", None),
        (PF_OPTION, "frame", "Choose frame format", 0,
                     ["square","round","half round",
                      "0.2 rounded square","0.1 rounded square"
                      ]),
        (PF_SPINNER, "size_fr",    "Frame thickness", 20, (0, 50, 2)),					  
        (PF_COLOR,   "color_frame", "Color frame", (200,200,200)),					  
        (PF_OPTION, "baseeffect", "Choose GMIC base effect ", 0,
                     ["Gradient Normalize","Afre Edge",
                      "Edges","Gradient Norm JR",
                      "Thin_Edges (very slow)",
                      "Convolve Prewitt X","Convolve Prewitt Y",
                      "Convolve Sobel X","Convolve Sobel Y",
                      "Convolve Potinv X","Convolve Potinv Y",
                      "Convolve Laplacian",
                      "Convolve Cross 1","Convolve Cross 2",
                      "Convolve Prewitt X+Y",
                      "Convolve Sobel X+Y",
                      "Convolve Potinv X+Y",
                      "Convolve Cross 1+2",
                      ]),
        (PF_TOGGLE, "getcolours", "Regain colours for final outcome?", True),
        (PF_TOGGLE, "thinlines", "Reduce width of blacks for final outcome?", False),
        (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ],
    [],
    LineartFramed,
    menu=menu
)

main()
